@@ -19,6 +19,8 @@
 #include "config.h"
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
+#include "nautilus-application.h"
+#include "nautilus-enum-types.h"
 
 #include <gio/gio.h>
 
@@ -29,14 +31,6 @@
  * instead of including gtk.h
  */
 #ifdef GTK_COMPILATION
-#include "gtkbutton.h"
-#include "gtkeventbox.h"
-#include "gtkimage.h"
-#include "gtkintl.h"
-#include "gtklabel.h"
-#include "gtkspinner.h"
-#include "gtkstack.h"
-#include "gtktypebuiltins.h"
 #else
 #include <gtk/gtk.h>
 #endif
@@ -50,7 +44,6 @@ struct _NautilusGtkPlacesViewRow
   GtkSpinner    *busy_spinner;
   GtkButton     *eject_button;
   GtkImage      *eject_icon;
-  GtkEventBox   *event_box;
   GtkImage      *icon_image;
   GtkLabel      *name_label;
   GtkLabel      *path_label;
@@ -61,7 +54,7 @@ struct _NautilusGtkPlacesViewRow
 
   GCancellable  *cancellable;
 
-  gint           is_network : 1;
+  int            is_network : 1;
 };
 
 G_DEFINE_TYPE (NautilusGtkPlacesViewRow, nautilus_gtk_places_view_row, GTK_TYPE_LIST_BOX_ROW)
@@ -90,9 +83,9 @@ measure_available_space_finished (GObject      *object,
   GError *error;
   guint64 free_space;
   guint64 total_space;
-  gchar *formatted_free_size;
-  gchar *formatted_total_size;
-  gchar *label;
+  char *formatted_free_size;
+  char *formatted_total_size;
+  char *label;
   guint plural_form;
 
   error = NULL;
@@ -149,9 +142,20 @@ out:
 static void
 measure_available_space (NautilusGtkPlacesViewRow *row)
 {
+  gboolean skip_measure;
   gboolean should_measure;
+  g_autoptr (GFile) root = NULL;
 
-  should_measure = (!row->is_network && (row->volume || row->mount || row->file));
+  skip_measure = FALSE;
+  if (nautilus_application_is_sandboxed ())
+    {
+      root = g_file_new_for_uri ("file:///");
+      if (row->file != NULL)
+        skip_measure = g_file_equal (root, row->file);
+    }
+
+  should_measure = ((row->volume || row->mount || row->file) &&
+                    !row->is_network && !skip_measure);
 
   gtk_label_set_label (row->available_space_label, "");
   gtk_widget_set_visible (GTK_WIDGET (row->available_space_label), should_measure);
@@ -218,16 +222,13 @@ nautilus_gtk_places_view_row_get_property (GObject    *object,
                                   GParamSpec *pspec)
 {
   NautilusGtkPlacesViewRow *self;
-  GIcon *icon;
 
   self = NAUTILUS_GTK_PLACES_VIEW_ROW (object);
-  icon = NULL;
 
   switch (prop_id)
     {
     case PROP_ICON:
-      gtk_image_get_gicon (self->icon_image, &icon, NULL);
-      g_value_set_object (value, icon);
+      g_value_set_object (value, gtk_image_get_gicon (self->icon_image));
       break;
 
     case PROP_NAME:
@@ -270,9 +271,7 @@ nautilus_gtk_places_view_row_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_ICON:
-      gtk_image_set_from_gicon (self->icon_image,
-                                g_value_get_object (value),
-                                GTK_ICON_SIZE_LARGE_TOOLBAR);
+      gtk_image_set_from_gicon (self->icon_image, g_value_get_object (value));
       break;
 
     case PROP_NAME:
@@ -316,6 +315,19 @@ nautilus_gtk_places_view_row_set_property (GObject      *object,
     }
 }
 
+static void
+nautilus_gtk_places_view_row_size_allocate (GtkWidget *widget,
+                                   int        width,
+                                   int        height,
+                                   int        baseline)
+{
+  GtkWidget *menu = GTK_WIDGET (g_object_get_data (G_OBJECT (widget), "menu"));
+
+  GTK_WIDGET_CLASS (nautilus_gtk_places_view_row_parent_class)->size_allocate (widget, width, height, baseline);
+  if (menu)
+    gtk_popover_present (GTK_POPOVER (menu));
+}
+
 static void
 nautilus_gtk_places_view_row_class_init (NautilusGtkPlacesViewRowClass *klass)
 {
@@ -326,6 +338,8 @@ nautilus_gtk_places_view_row_class_init (NautilusGtkPlacesViewRowClass *klass)
   object_class->get_property = nautilus_gtk_places_view_row_get_property;
   object_class->set_property = nautilus_gtk_places_view_row_set_property;
 
+  widget_class->size_allocate = nautilus_gtk_places_view_row_size_allocate;
+
   properties[PROP_ICON] =
           g_param_spec_object ("icon",
                                "Icon of the row",
@@ -384,7 +398,6 @@ nautilus_gtk_places_view_row_class_init (NautilusGtkPlacesViewRowClass *klass)
   gtk_widget_class_bind_template_child (widget_class, NautilusGtkPlacesViewRow, busy_spinner);
   gtk_widget_class_bind_template_child (widget_class, NautilusGtkPlacesViewRow, eject_button);
   gtk_widget_class_bind_template_child (widget_class, NautilusGtkPlacesViewRow, eject_icon);
-  gtk_widget_class_bind_template_child (widget_class, NautilusGtkPlacesViewRow, event_box);
   gtk_widget_class_bind_template_child (widget_class, NautilusGtkPlacesViewRow, icon_image);
   gtk_widget_class_bind_template_child (widget_class, NautilusGtkPlacesViewRow, name_label);
   gtk_widget_class_bind_template_child (widget_class, NautilusGtkPlacesViewRow, path_label);
@@ -438,14 +451,6 @@ nautilus_gtk_places_view_row_get_eject_button (NautilusGtkPlacesViewRow *row)
   return GTK_WIDGET (row->eject_button);
 }
 
-GtkWidget*
-nautilus_gtk_places_view_row_get_event_box (NautilusGtkPlacesViewRow *row)
-{
-  g_return_val_if_fail (NAUTILUS_IS_GTK_PLACES_VIEW_ROW (row), NULL);
-
-  return GTK_WIDGET (row->event_box);
-}
-
 void
 nautilus_gtk_places_view_row_set_busy (NautilusGtkPlacesViewRow *row,
                               gboolean          is_busy)
@@ -456,10 +461,12 @@ nautilus_gtk_places_view_row_set_busy (NautilusGtkPlacesViewRow *row,
     {
       gtk_stack_set_visible_child (row->mount_stack, GTK_WIDGET (row->busy_spinner));
       gtk_widget_set_child_visible (GTK_WIDGET (row->mount_stack), TRUE);
+      gtk_spinner_start (row->busy_spinner);
     }
   else
     {
       gtk_widget_set_child_visible (GTK_WIDGET (row->mount_stack), FALSE);
+      gtk_spinner_stop (row->busy_spinner);
     }
 }
 
@@ -479,7 +486,7 @@ nautilus_gtk_places_view_row_set_is_network (NautilusGtkPlacesViewRow *row,
     {
       row->is_network = is_network;
 
-      gtk_image_set_from_icon_name (row->eject_icon, "media-eject-symbolic", GTK_ICON_SIZE_BUTTON);
+      gtk_image_set_from_icon_name (row->eject_icon, "media-eject-symbolic");
       gtk_widget_set_tooltip_text (GTK_WIDGET (row->eject_button), is_network ? _("Disconnect") : _("Unmount"));
     }
 }
