@@ -26,7 +26,6 @@
 #include <gio/gio.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgd/gd.h>
 #include <string.h>
 
 #include "nautilus-file.h"
@@ -36,19 +35,21 @@
 #include "nautilus-search-popover.h"
 #include "nautilus-mime-actions.h"
 #include "nautilus-ui-utilities.h"
-#include "nautilus-gtk4-helpers.h"
 
 struct _NautilusQueryEditor
 {
-    GtkBox parent_instance;
+    GtkWidget parent_instance;
 
-    GtkWidget *entry;
+    GtkWidget *tags_box;
+    GtkWidget *text;
+    GtkWidget *clear_icon;
     GtkWidget *popover;
     GtkWidget *dropdown_button;
 
-    GdTaggedEntryTag *mime_types_tag;
-    GdTaggedEntryTag *date_range_tag;
+    GtkWidget *mime_types_tag;
+    GtkWidget *date_range_tag;
 
+    guint search_changed_timeout_id;
     gboolean change_frozen;
 
     GFile *location;
@@ -81,7 +82,13 @@ static void entry_changed_cb (GtkWidget           *entry,
                               NautilusQueryEditor *editor);
 static void nautilus_query_editor_changed (NautilusQueryEditor *editor);
 
-G_DEFINE_TYPE (NautilusQueryEditor, nautilus_query_editor, GTK_TYPE_BOX);
+G_DEFINE_TYPE (NautilusQueryEditor, nautilus_query_editor, GTK_TYPE_WIDGET);
+
+/* A hunt-and-peck typist types at 25-35 words per minute, which means 342 to 480ms between strokes.
+ * An average touch typist types at 50-70 wpm, which means 171 to 240ms "under ideal conditions".
+ * A 150ms default search triggering delay is too short even for fast typists in general,
+ * so wait 400ms after typing, to improve performance by not spamming search engines: */
+#define SEARCH_CHANGED_TIMEOUT 400
 
 static void
 update_fts_sensitivity (NautilusQueryEditor *editor)
@@ -135,6 +142,14 @@ nautilus_query_editor_dispose (GObject *object)
 
     editor = NAUTILUS_QUERY_EDITOR (object);
 
+    g_clear_handle_id (&editor->search_changed_timeout_id, g_source_remove);
+
+    gtk_widget_unparent (gtk_widget_get_first_child (GTK_WIDGET (editor)));
+    g_clear_pointer (&editor->tags_box, gtk_widget_unparent);
+    g_clear_pointer (&editor->text, gtk_widget_unparent);
+    g_clear_pointer (&editor->dropdown_button, gtk_widget_unparent);
+    g_clear_pointer (&editor->clear_icon, gtk_widget_unparent);
+
     g_clear_object (&editor->location);
     g_clear_object (&editor->query);
 
@@ -145,19 +160,19 @@ nautilus_query_editor_dispose (GObject *object)
     G_OBJECT_CLASS (nautilus_query_editor_parent_class)->dispose (object);
 }
 
-static void
+static gboolean
 nautilus_query_editor_grab_focus (GtkWidget *widget)
 {
     NautilusQueryEditor *editor;
 
     editor = NAUTILUS_QUERY_EDITOR (widget);
 
-    if (gtk_widget_get_visible (widget) && !gtk_widget_is_focus (editor->entry))
+    if (gtk_widget_get_visible (widget) && !gtk_widget_is_focus (editor->text))
     {
-        /* avoid selecting the entry text */
-        gtk_widget_grab_focus (editor->entry);
-        gtk_editable_set_position (GTK_EDITABLE (editor->entry), -1);
+        return gtk_text_grab_focus_without_selecting (GTK_TEXT (editor->text));
     }
+
+    return FALSE;
 }
 
 static void
@@ -225,13 +240,6 @@ nautilus_query_editor_set_property (GObject      *object,
 static void
 nautilus_query_editor_finalize (GObject *object)
 {
-    NautilusQueryEditor *editor;
-
-    editor = NAUTILUS_QUERY_EDITOR (object);
-
-    g_clear_object (&editor->date_range_tag);
-    g_clear_object (&editor->mime_types_tag);
-
     G_OBJECT_CLASS (nautilus_query_editor_parent_class)->finalize (object);
 }
 
@@ -240,6 +248,7 @@ nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
 {
     GObjectClass *gobject_class;
     GtkWidgetClass *widget_class;
+    g_autoptr (GtkShortcut) shortcut = NULL;
 
     gobject_class = G_OBJECT_CLASS (class);
     gobject_class->finalize = nautilus_query_editor_finalize;
@@ -286,11 +295,13 @@ nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
                       g_cclosure_marshal_VOID__VOID,
                       G_TYPE_NONE, 0);
 
-    gtk_binding_entry_add_signal (gtk_binding_set_by_class (class),
-                                  GDK_KEY_Down,
-                                  0,
-                                  "focus-view",
-                                  0);
+    shortcut = gtk_shortcut_new (gtk_keyval_trigger_new (GDK_KEY_Down, 0),
+                                 gtk_signal_action_new ("focus-view"));
+    gtk_widget_class_add_shortcut (widget_class, shortcut);
+
+    gtk_widget_class_add_binding_signal (widget_class,
+                                         GDK_KEY_Escape, 0, "cancel",
+                                         NULL);
 
     /**
      * NautilusQueryEditor::location:
@@ -318,6 +329,10 @@ nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
                                                           "The query that the editor is handling",
                                                           NAUTILUS_TYPE_QUERY,
                                                           G_PARAM_READWRITE));
+
+    gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BOX_LAYOUT);
+    gtk_widget_class_set_css_name (widget_class, "entry");
+    gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_SEARCH_BOX);
 }
 
 GFile *
@@ -349,7 +364,7 @@ create_query (NautilusQueryEditor *editor)
 
     nautilus_query_set_search_content (query, fts_enabled);
 
-    nautilus_query_set_text (query, gtk_entry_get_text (GTK_ENTRY (editor->entry)));
+    nautilus_query_set_text (query, gtk_editable_get_text (GTK_EDITABLE (editor->text)));
     nautilus_query_set_location (query, editor->location);
 
     /* We only set the query using the global setting for recursivity here,
@@ -367,6 +382,31 @@ entry_activate_cb (GtkWidget           *entry,
     g_signal_emit (editor, signals[ACTIVATED], 0);
 }
 
+static gboolean
+entry_changed_internal (NautilusQueryEditor *editor)
+{
+    const gchar *text = gtk_editable_get_text (GTK_EDITABLE (editor->text));
+    gboolean is_empty = (text == NULL && *text == '\0');
+
+    editor->search_changed_timeout_id = 0;
+
+    gtk_widget_set_child_visible (editor->clear_icon, !is_empty);
+
+    if (editor->query == NULL)
+    {
+        create_query (editor);
+    }
+    else
+    {
+        g_autofree gchar *stripped_text = g_strstrip (g_strdup (text));
+        nautilus_query_set_text (editor->query, stripped_text);
+    }
+
+    nautilus_query_editor_changed (editor);
+
+    return G_SOURCE_REMOVE;
+}
+
 static void
 entry_changed_cb (GtkWidget           *entry,
                   NautilusQueryEditor *editor)
@@ -376,39 +416,39 @@ entry_changed_cb (GtkWidget           *entry,
         return;
     }
 
-    if (editor->query == NULL)
-    {
-        create_query (editor);
-    }
-    else
-    {
-        g_autofree gchar *text = NULL;
-
-        text = g_strdup (gtk_entry_get_text (GTK_ENTRY (editor->entry)));
-        text = g_strstrip (text);
-
-        nautilus_query_set_text (editor->query, text);
-    }
-
-    nautilus_query_editor_changed (editor);
+    g_clear_handle_id (&editor->search_changed_timeout_id, g_source_remove);
+    editor->search_changed_timeout_id = g_timeout_add (SEARCH_CHANGED_TIMEOUT,
+                                                       G_SOURCE_FUNC (entry_changed_internal),
+                                                       editor);
 }
 
-static void
-nautilus_query_editor_on_stop_search (GtkWidget           *entry,
-                                      NautilusQueryEditor *editor)
+static GtkWidget *
+create_tag (NautilusQueryEditor *self,
+            const gchar         *text,
+            GCallback            reset_callback)
 {
-    g_signal_emit (editor, signals[CANCEL], 0);
-}
+    GtkWidget *tag;
+    GtkWidget *label;
+    GtkWidget *button;
 
-/* Type */
+    tag = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
+    gtk_widget_set_margin_end (tag, 6);
+    gtk_widget_set_name (tag, "NautilusQueryEditorTag");
 
-static void
-nautilus_query_editor_init (NautilusQueryEditor *editor)
-{
-    g_signal_connect (nautilus_preferences,
-                      "changed::recursive-search",
-                      G_CALLBACK (recursive_search_preferences_changed),
-                      editor);
+    label = gtk_label_new (text);
+    gtk_widget_add_css_class (label, "caption-heading");
+    gtk_widget_set_margin_start (label, 12);
+    gtk_box_append (GTK_BOX (tag), label);
+
+    button = gtk_button_new ();
+    gtk_button_set_icon_name (GTK_BUTTON (button), "window-close-symbolic");
+    gtk_widget_add_css_class (button, "flat");
+    gtk_widget_add_css_class (button, "circular");
+    g_signal_connect_object (button, "clicked",
+                             reset_callback, self->popover, G_CONNECT_SWAPPED);
+    gtk_box_append (GTK_BOX (tag), button);
+
+    return tag;
 }
 
 static void
@@ -425,22 +465,28 @@ search_popover_date_range_changed_cb (NautilusSearchPopover *popover,
         create_query (editor);
     }
 
-    gd_tagged_entry_remove_tag (GD_TAGGED_ENTRY (editor->entry),
-                                editor->date_range_tag);
+    if (editor->date_range_tag != NULL)
+    {
+        gtk_box_remove (GTK_BOX (editor->tags_box), editor->date_range_tag);
+        editor->date_range_tag = NULL;
+    }
+
     if (date_range)
     {
         g_autofree gchar *text_for_date_range = NULL;
 
         text_for_date_range = get_text_for_date_range (date_range, TRUE);
-        gd_tagged_entry_tag_set_label (editor->date_range_tag,
-                                       text_for_date_range);
-        gd_tagged_entry_add_tag (GD_TAGGED_ENTRY (editor->entry),
-                                 GD_TAGGED_ENTRY_TAG (editor->date_range_tag));
+        editor->date_range_tag = create_tag (editor,
+                                             text_for_date_range,
+                                             G_CALLBACK (nautilus_search_popover_reset_date_range));
+        gtk_box_append (GTK_BOX (editor->tags_box), editor->date_range_tag);
     }
 
     nautilus_query_set_date_range (editor->query, date_range);
 
     nautilus_query_editor_changed (editor);
+    gtk_widget_set_visible (editor->tags_box,
+                            (gtk_widget_get_first_child (editor->tags_box) != NULL));
 }
 
 static void
@@ -459,8 +505,12 @@ search_popover_mime_type_changed_cb (NautilusSearchPopover *popover,
         create_query (editor);
     }
 
-    gd_tagged_entry_remove_tag (GD_TAGGED_ENTRY (editor->entry),
-                                editor->mime_types_tag);
+    if (editor->mime_types_tag != NULL)
+    {
+        gtk_box_remove (GTK_BOX (editor->tags_box), editor->mime_types_tag);
+        editor->mime_types_tag = NULL;
+    }
+
     /* group 0 is anything */
     if (mimetype_group == 0)
     {
@@ -469,10 +519,10 @@ search_popover_mime_type_changed_cb (NautilusSearchPopover *popover,
     else if (mimetype_group > 0)
     {
         mimetypes = nautilus_mime_types_group_get_mimetypes (mimetype_group);
-        gd_tagged_entry_tag_set_label (editor->mime_types_tag,
-                                       nautilus_mime_types_group_get_name (mimetype_group));
-        gd_tagged_entry_add_tag (GD_TAGGED_ENTRY (editor->entry),
-                                 GD_TAGGED_ENTRY_TAG (editor->mime_types_tag));
+        editor->mime_types_tag = create_tag (editor,
+                                             nautilus_mime_types_group_get_name (mimetype_group),
+                                             G_CALLBACK (nautilus_search_popover_reset_mime_types));
+        gtk_box_append (GTK_BOX (editor->tags_box), editor->mime_types_tag);
     }
     else
     {
@@ -480,14 +530,18 @@ search_popover_mime_type_changed_cb (NautilusSearchPopover *popover,
 
         mimetypes = g_ptr_array_new_full (1, g_free);
         g_ptr_array_add (mimetypes, g_strdup (mimetype));
+
         display_name = g_content_type_get_description (mimetype);
-        gd_tagged_entry_tag_set_label (editor->mime_types_tag, display_name);
-        gd_tagged_entry_add_tag (GD_TAGGED_ENTRY (editor->entry),
-                                 GD_TAGGED_ENTRY_TAG (editor->mime_types_tag));
+        editor->mime_types_tag = create_tag (editor,
+                                             display_name,
+                                             G_CALLBACK (nautilus_search_popover_reset_mime_types));
+        gtk_box_append (GTK_BOX (editor->tags_box), editor->mime_types_tag);
     }
     nautilus_query_set_mime_types (editor->query, mimetypes);
 
     nautilus_query_editor_changed (editor);
+    gtk_widget_set_visible (editor->tags_box,
+                            (gtk_widget_get_first_child (editor->tags_box) != NULL));
 }
 
 static void
@@ -530,58 +584,66 @@ search_popover_fts_changed_cb (GObject    *popover,
 }
 
 static void
-entry_tag_clicked (NautilusQueryEditor *editor)
+on_clear_icon_pressed (GtkGestureClick     *gesture,
+                       int                  n_press,
+                       double               x,
+                       double               y,
+                       NautilusQueryEditor *self)
 {
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (editor->dropdown_button),
-                                  TRUE);
+    gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
 }
 
 static void
-entry_tag_close_button_clicked (NautilusQueryEditor *editor,
-                                GdTaggedEntryTag    *tag)
+on_clear_icon_released (GtkGestureClick     *gesture,
+                        int                  n_press,
+                        double               x,
+                        double               y,
+                        NautilusQueryEditor *self)
 {
-    if (tag == editor->mime_types_tag)
-    {
-        nautilus_search_popover_reset_mime_types (NAUTILUS_SEARCH_POPOVER (editor->popover));
-    }
-    else
-    {
-        nautilus_search_popover_reset_date_range (NAUTILUS_SEARCH_POPOVER (editor->popover));
-    }
+    gtk_editable_set_text (GTK_EDITABLE (self->text), "");
 }
 
 static void
-setup_widgets (NautilusQueryEditor *editor)
+nautilus_query_editor_init (NautilusQueryEditor *editor)
 {
-    GtkWidget *hbox;
-    GtkWidget *vbox;
+    gboolean rtl = (gtk_widget_get_direction (GTK_WIDGET (editor)) == GTK_TEXT_DIR_RTL);
+    GtkWidget *image;
+    GtkEventController *controller;
 
-    /* vertical box that holds the search entry and the label below */
-    vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
-    gtk_box_append (GTK_BOX (editor), vbox);
+    gtk_widget_set_name (GTK_WIDGET (editor), "NautilusQueryEditor");
+    gtk_widget_add_css_class (GTK_WIDGET (editor), "search");
 
-    /* horizontal box */
-    hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
-    gtk_style_context_add_class (gtk_widget_get_style_context (hbox), "linked");
-    gtk_box_append (GTK_BOX (vbox), hbox);
+    g_signal_connect (nautilus_preferences,
+                      "changed::recursive-search",
+                      G_CALLBACK (recursive_search_preferences_changed),
+                      editor);
 
     /* create the search entry */
-    editor->entry = GTK_WIDGET (gd_tagged_entry_new ());
-    gtk_widget_set_hexpand (editor->entry, TRUE);
+    image = gtk_image_new_from_icon_name ("system-search-symbolic");
+    g_object_set (image, "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION, NULL);
+    gtk_widget_set_margin_start (image, 4);
+    gtk_widget_set_margin_end (image, 6);
+    gtk_widget_set_parent (image, GTK_WIDGET (editor));
 
-    gtk_box_append (GTK_BOX (hbox), editor->entry);
+    editor->tags_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
+    gtk_widget_hide (editor->tags_box);
+    gtk_widget_set_parent (editor->tags_box, GTK_WIDGET (editor));
 
-    editor->mime_types_tag = gd_tagged_entry_tag_new (NULL);
-    editor->date_range_tag = gd_tagged_entry_tag_new (NULL);
+    editor->text = gtk_text_new ();
+    gtk_widget_set_hexpand (editor->text, TRUE);
+    gtk_widget_set_parent (editor->text, GTK_WIDGET (editor));
 
-    g_signal_connect_swapped (editor->entry,
-                              "tag-clicked",
-                              G_CALLBACK (entry_tag_clicked),
-                              editor);
-    g_signal_connect_swapped (editor->entry,
-                              "tag-button-clicked",
-                              G_CALLBACK (entry_tag_close_button_clicked),
-                              editor);
+    editor->clear_icon = gtk_image_new_from_icon_name (rtl ? "edit-clear-rtl-symbolic" :
+                                                             "edit-clear-symbolic");
+    g_object_set (editor->clear_icon, "accessible-role", GTK_ACCESSIBLE_ROLE_PRESENTATION, NULL);
+    gtk_widget_set_tooltip_text (editor->clear_icon, _("Clear entry"));
+    gtk_widget_set_child_visible (editor->clear_icon, FALSE);
+    gtk_widget_set_parent (editor->clear_icon, GTK_WIDGET (editor));
+
+    controller = GTK_EVENT_CONTROLLER (gtk_gesture_click_new ());
+    g_signal_connect (controller, "pressed", G_CALLBACK (on_clear_icon_pressed), editor);
+    g_signal_connect (controller, "released", G_CALLBACK (on_clear_icon_released), editor);
+    gtk_widget_add_controller (editor->clear_icon, controller);
 
     /* setup the search popover */
     editor->popover = nautilus_search_popover_new ();
@@ -597,15 +659,15 @@ setup_widgets (NautilusQueryEditor *editor)
 
     /* setup the filter menu button */
     editor->dropdown_button = gtk_menu_button_new ();
+    gtk_menu_button_set_icon_name (GTK_MENU_BUTTON (editor->dropdown_button), "funnel-symbolic");
     gtk_menu_button_set_popover (GTK_MENU_BUTTON (editor->dropdown_button), editor->popover);
-    gtk_box_append (GTK_BOX (hbox), editor->dropdown_button);
+    gtk_widget_set_parent (editor->dropdown_button, GTK_WIDGET (editor));
+    gtk_widget_add_css_class (editor->dropdown_button, "circular");
 
-    g_signal_connect (editor->entry, "activate",
+    g_signal_connect (editor->text, "activate",
                       G_CALLBACK (entry_activate_cb), editor);
-    g_signal_connect (editor->entry, "search-changed",
+    g_signal_connect (editor->text, "changed",
                       G_CALLBACK (entry_changed_cb), editor);
-    g_signal_connect (editor->entry, "stop-search",
-                      G_CALLBACK (nautilus_query_editor_on_stop_search), editor);
     g_signal_connect (editor->popover, "date-range",
                       G_CALLBACK (search_popover_date_range_changed_cb), editor);
     g_signal_connect (editor->popover, "mime-type",
@@ -614,9 +676,6 @@ setup_widgets (NautilusQueryEditor *editor)
                       G_CALLBACK (search_popover_time_type_changed_cb), editor);
     g_signal_connect (editor->popover, "notify::fts-enabled",
                       G_CALLBACK (search_popover_fts_changed_cb), editor);
-
-    /* show everything */
-    gtk_widget_show_all (vbox);
 }
 
 static void
@@ -635,7 +694,7 @@ nautilus_query_editor_get_query (NautilusQueryEditor *editor)
 {
     g_return_val_if_fail (NAUTILUS_IS_QUERY_EDITOR (editor), NULL);
 
-    if (editor->entry == NULL)
+    if (editor->text == NULL)
     {
         return NULL;
     }
@@ -646,13 +705,7 @@ nautilus_query_editor_get_query (NautilusQueryEditor *editor)
 GtkWidget *
 nautilus_query_editor_new (void)
 {
-    NautilusQueryEditor *editor;
-
-    editor = g_object_new (NAUTILUS_TYPE_QUERY_EDITOR, NULL);
-
-    setup_widgets (editor);
-
-    return GTK_WIDGET (editor);
+    return GTK_WIDGET (g_object_new (NAUTILUS_TYPE_QUERY_EDITOR, NULL));
 }
 
 void
@@ -719,11 +772,11 @@ nautilus_query_editor_set_query (NautilusQueryEditor *self,
 
     self->change_frozen = TRUE;
 
-    current_text = g_strdup (gtk_entry_get_text (GTK_ENTRY (self->entry)));
+    current_text = g_strdup (gtk_editable_get_text (GTK_EDITABLE (self->text)));
     current_text = g_strstrip (current_text);
     if (!g_str_equal (current_text, text))
     {
-        gtk_entry_set_text (GTK_ENTRY (self->entry), text);
+        gtk_editable_set_text (GTK_EDITABLE (self->text), text);
     }
 
     if (g_set_object (&self->query, query))
@@ -742,7 +795,7 @@ nautilus_query_editor_set_text (NautilusQueryEditor *self,
     g_return_if_fail (text != NULL);
 
     /* The handler of the entry will take care of everything */
-    gtk_entry_set_text (GTK_ENTRY (self->entry), text);
+    gtk_editable_set_text (GTK_EDITABLE (self->text), text);
 }
 
 static gboolean
@@ -758,7 +811,7 @@ nautilus_gtk_search_entry_is_keynav_event (guint           keyval,
         keyval == GDK_KEY_End || keyval == GDK_KEY_KP_End ||
         keyval == GDK_KEY_Page_Up || keyval == GDK_KEY_KP_Page_Up ||
         keyval == GDK_KEY_Page_Down || keyval == GDK_KEY_KP_Page_Down ||
-        ((state & (GDK_CONTROL_MASK | GDK_MOD1_MASK)) != 0))
+        ((state & (GDK_CONTROL_MASK | GDK_ALT_MASK)) != 0))
     {
         return TRUE;
     }
@@ -786,5 +839,5 @@ nautilus_query_editor_handle_event (NautilusQueryEditor   *self,
         return GDK_EVENT_PROPAGATE;
     }
 
-    return gtk_event_controller_key_forward (controller, GTK_WIDGET (self->entry));
+    return gtk_event_controller_key_forward (controller, self->text);
 }
