@@ -24,7 +24,6 @@
 #include "nautilus-file.h"
 #include "nautilus-ui-utilities.h"
 #include "nautilus-global-preferences.h"
-#include "nautilus-gtk4-helpers.h"
 
  #define SEARCH_FILTER_MAX_YEARS 5
 
@@ -51,7 +50,7 @@ struct _NautilusSearchPopover
     GtkWidget *filename_search_button;
 
     NautilusQuery *query;
-    GtkTreeView *treeview;
+    GtkSingleSelection *other_types_model;
 
     gboolean fts_enabled;
 };
@@ -92,12 +91,9 @@ calendar_day_selected (GtkCalendar           *calendar,
                        NautilusSearchPopover *popover)
 {
     GDateTime *date;
-    guint year, month, day;
     GPtrArray *date_range;
 
-    gtk_calendar_get_date (calendar, &year, &month, &day);
-
-    date = g_date_time_new_local (year, month + 1, day, 0, 0, 0);
+    date = gtk_calendar_get_date (calendar);
 
     date_range = g_ptr_array_new_full (2, (GDestroyNotify) g_date_time_unref);
     g_ptr_array_add (date_range, g_date_time_ref (date));
@@ -127,12 +123,7 @@ setup_date (NautilusSearchPopover *popover,
 
         g_signal_handlers_block_by_func (popover->calendar, calendar_day_selected, popover);
 
-        gtk_calendar_select_month (GTK_CALENDAR (popover->calendar),
-                                   g_date_time_get_month (date_initial) - 1,
-                                   g_date_time_get_year (date_initial));
-
-        gtk_calendar_select_day (GTK_CALENDAR (popover->calendar),
-                                 g_date_time_get_day_of_month (date_initial));
+        gtk_calendar_select_day (GTK_CALENDAR (popover->calendar), date_initial);
 
         update_date_label (popover, date_range);
 
@@ -166,7 +157,7 @@ date_entry_activate (GtkEntry              *entry,
         GDate *date;
 
         date = g_date_new ();
-        g_date_set_parse (date, gtk_entry_get_text (entry));
+        g_date_set_parse (date, gtk_editable_get_text (GTK_EDITABLE (entry)));
 
         /* Invalid date silently does nothing */
         if (!g_date_valid (date))
@@ -287,7 +278,6 @@ select_type_button_clicked (GtkButton             *button,
 static void
 toggle_calendar_icon_clicked (GtkEntry              *entry,
                               GtkEntryIconPosition   position,
-                              GdkEvent              *event,
                               NautilusSearchPopover *popover)
 {
     const gchar *current_visible_child;
@@ -343,16 +333,16 @@ types_listbox_row_activated (GtkListBox            *listbox,
 }
 
 static void
-search_time_type_changed (GtkToggleButton       *button,
+search_time_type_changed (GtkCheckButton        *button,
                           NautilusSearchPopover *popover)
 {
     NautilusQuerySearchType type = -1;
 
-    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (popover->last_modified_button)))
+    if (gtk_check_button_get_active (GTK_CHECK_BUTTON (popover->last_modified_button)))
     {
         type = NAUTILUS_QUERY_SEARCH_TYPE_LAST_MODIFIED;
     }
-    else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (popover->last_used_button)))
+    else if (gtk_check_button_get_active (GTK_CHECK_BUTTON (popover->last_used_button)))
     {
         type = NAUTILUS_QUERY_SEARCH_TYPE_LAST_ACCESS;
     }
@@ -407,7 +397,7 @@ create_row_for_label (const gchar *text,
                           NULL);
 
     gtk_list_box_row_set_child (GTK_LIST_BOX_ROW (row), label);
-    gtk_widget_show_all (row);
+    gtk_widget_show (row);
 
     return row;
 }
@@ -552,20 +542,13 @@ on_other_types_dialog_response (GtkDialog             *dialog,
 {
     if (response_id == GTK_RESPONSE_OK)
     {
-        GtkTreeIter iter;
-        GtkTreeSelection *selection;
-        GtkTreeModel *store;
-        char *mimetype;
-        char *description;
+        GtkStringObject *item;
+        const char *mimetype;
+        g_autofree gchar *description = NULL;
 
-        store = gtk_tree_view_get_model (popover->treeview);
-        selection = gtk_tree_view_get_selection (popover->treeview);
-
-        gtk_tree_selection_get_selected (selection, NULL, &iter);
-        gtk_tree_model_get (store, &iter,
-                            0, &description,
-                            1, &mimetype,
-                            -1);
+        item = gtk_single_selection_get_selected_item (popover->other_types_model);
+        mimetype = gtk_string_object_get_string (item);
+        description = g_content_type_get_description (mimetype);
 
         gtk_label_set_label (GTK_LABEL (popover->type_label), description);
 
@@ -574,88 +557,131 @@ on_other_types_dialog_response (GtkDialog             *dialog,
         gtk_stack_set_visible_child_name (GTK_STACK (popover->type_stack), "type-button");
     }
 
-    g_clear_object (&popover->treeview);
-    gtk_widget_destroy (GTK_WIDGET (dialog));
+    g_clear_object (&popover->other_types_model);
+    gtk_window_destroy (GTK_WINDOW (dialog));
+}
+
+static void
+on_other_types_activate (GtkListView *self,
+                         guint        position,
+                         gpointer     user_data)
+{
+    GtkDialog *dialog = GTK_DIALOG (user_data);
+
+    gtk_dialog_response (dialog, GTK_RESPONSE_OK);
+}
+
+static void
+on_other_types_bind (GtkSignalListItemFactory *factory,
+                     GtkListItem              *listitem,
+                     gpointer                  user_data)
+{
+    GtkLabel *label;
+    GtkStringObject *item;
+    g_autofree gchar *description = NULL;
+
+    label = GTK_LABEL (gtk_list_item_get_child (listitem));
+    item = GTK_STRING_OBJECT (gtk_list_item_get_item (listitem));
+
+    description = g_content_type_get_description (gtk_string_object_get_string (item));
+    gtk_label_set_text (label, description);
+}
+
+static void
+on_other_types_setup (GtkSignalListItemFactory *factory,
+                      GtkListItem              *listitem,
+                      gpointer                  user_data)
+{
+    GtkWidget *label;
+
+    label = gtk_label_new (NULL);
+    gtk_widget_set_halign (label, GTK_ALIGN_START);
+    gtk_widget_set_margin_start (label, 12);
+    gtk_widget_set_margin_end (label, 12);
+    gtk_widget_set_margin_top (label, 6);
+    gtk_widget_set_margin_bottom (label, 6);
+    gtk_list_item_set_child (listitem, label);
 }
 
 static void
 show_other_types_dialog (NautilusSearchPopover *popover)
 {
-    GList *mime_infos, *l;
+    GtkStringList *string_model;
+    GtkStringSorter *sorter;
+    GtkSortListModel *sort_model;
+    GtkStringFilter *filter;
+    GtkFilterListModel *filter_model;
+    g_autoptr (GList) mime_infos = NULL;
     GtkWidget *dialog;
-    GtkWidget *scrolled, *treeview;
-    GtkListStore *store;
-    GtkTreeViewColumn *column;
-    GtkCellRenderer *renderer;
-    GtkWidget *toplevel;
-    GtkTreeSelection *selection;
+    GtkWidget *content_area;
+    GtkWidget *search_entry;
+    GtkWidget *scrolled;
+    GtkListItemFactory *factory;
+    GtkWidget *listview;
+    GtkRoot *toplevel;
 
+    string_model = gtk_string_list_new (NULL);
     mime_infos = g_content_types_get_registered ();
-
-    store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
-    for (l = mime_infos; l != NULL; l = l->next)
+    for (GList *l = mime_infos; l != NULL; l = l->next)
     {
-        GtkTreeIter iter;
-        char *mime_type = l->data;
-        char *description;
-
-        description = g_content_type_get_description (mime_type);
-        if (description == NULL)
-        {
-            description = g_strdup (mime_type);
-        }
-
-        gtk_list_store_append (store, &iter);
-        gtk_list_store_set (store, &iter,
-                            0, description,
-                            1, mime_type,
-                            -1);
-
-        g_free (mime_type);
-        g_free (description);
+        gtk_string_list_append (string_model, l->data);
     }
-    g_list_free (mime_infos);
+    sorter = gtk_string_sorter_new (gtk_property_expression_new (GTK_TYPE_STRING_OBJECT, NULL, "string"));
+    sort_model = gtk_sort_list_model_new (G_LIST_MODEL (string_model), GTK_SORTER (sorter));
+    filter = gtk_string_filter_new (gtk_property_expression_new (GTK_TYPE_STRING_OBJECT, NULL, "string"));
+    filter_model = gtk_filter_list_model_new (G_LIST_MODEL (sort_model), GTK_FILTER (filter));
+    popover->other_types_model = gtk_single_selection_new (G_LIST_MODEL (filter_model));
 
-    toplevel = gtk_widget_get_toplevel (GTK_WIDGET (popover));
+    toplevel = gtk_widget_get_root (GTK_WIDGET (popover));
     dialog = gtk_dialog_new_with_buttons (_("Select type"),
                                           GTK_WINDOW (toplevel),
                                           GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_USE_HEADER_BAR,
                                           _("_Cancel"), GTK_RESPONSE_CANCEL,
                                           _("Select"), GTK_RESPONSE_OK,
                                           NULL);
+    gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
     gtk_window_set_default_size (GTK_WINDOW (dialog), 400, 600);
 
-    scrolled = gtk_scrolled_window_new (NULL, NULL);
+    /* If there are 0 results, make action insensitive */
+    g_object_bind_property (filter_model,
+                            "n-items",
+                            gtk_dialog_get_widget_for_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK),
+                            "sensitive",
+                            G_BINDING_DEFAULT);
+
+    content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
+
+    search_entry = gtk_search_entry_new ();
+    gtk_search_entry_set_key_capture_widget (GTK_SEARCH_ENTRY (search_entry), content_area);
+    g_object_bind_property (search_entry, "text", filter, "search", G_BINDING_SYNC_CREATE);
+    gtk_box_append (GTK_BOX (content_area), search_entry);
+    gtk_widget_set_margin_start (search_entry, 12);
+    gtk_widget_set_margin_end (search_entry, 12);
+    gtk_widget_set_margin_top (search_entry, 6);
+    gtk_widget_set_margin_bottom (search_entry, 6);
+
+    gtk_box_append (GTK_BOX (content_area), gtk_separator_new (GTK_ORIENTATION_VERTICAL));
+
+    scrolled = gtk_scrolled_window_new ();
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
                                     GTK_POLICY_AUTOMATIC,
                                     GTK_POLICY_AUTOMATIC);
 
     gtk_widget_set_vexpand (scrolled, TRUE);
-    gtk_widget_show (scrolled);
-    gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), scrolled, FALSE, TRUE, 0);
+    gtk_box_append (GTK_BOX (content_area), scrolled);
 
-    treeview = gtk_tree_view_new ();
-    gtk_tree_view_set_model (GTK_TREE_VIEW (treeview), GTK_TREE_MODEL (store));
-    gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), 0, GTK_SORT_ASCENDING);
+    factory = gtk_signal_list_item_factory_new ();
+    g_signal_connect (factory, "setup", G_CALLBACK (on_other_types_setup), NULL);
+    g_signal_connect (factory, "bind", G_CALLBACK (on_other_types_bind), NULL);
 
-    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
-    gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
+    listview = gtk_list_view_new (GTK_SELECTION_MODEL (g_object_ref (popover->other_types_model)),
+                                  factory);
+    g_signal_connect (listview, "activate", G_CALLBACK (on_other_types_activate), dialog);
 
-    renderer = gtk_cell_renderer_text_new ();
-    column = gtk_tree_view_column_new_with_attributes ("Name",
-                                                       renderer,
-                                                       "text",
-                                                       0,
-                                                       NULL);
-    gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
-    gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
-
-    gtk_widget_show (treeview);
-    gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (scrolled), treeview);
-    popover->treeview = GTK_TREE_VIEW (g_object_ref (treeview));
+    gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (scrolled), listview);
 
     g_signal_connect (dialog, "response", G_CALLBACK (on_other_types_dialog_response), popover);
-    gtk_widget_show_all (dialog);
+    gtk_widget_show (dialog);
 }
 
 static void
@@ -677,7 +703,7 @@ update_date_label (NautilusSearchPopover *popover,
 
         label = get_text_for_date_range (date_range, TRUE);
 
-        gtk_entry_set_text (GTK_ENTRY (popover->date_entry), days < 1 ? label : "");
+        gtk_editable_set_text (GTK_EDITABLE (popover->date_entry), days < 1 ? label : "");
 
         gtk_widget_show (popover->clear_date_button);
         gtk_label_set_label (GTK_LABEL (popover->select_date_button_label), label);
@@ -689,7 +715,7 @@ update_date_label (NautilusSearchPopover *popover,
     {
         gtk_label_set_label (GTK_LABEL (popover->select_date_button_label),
                              _("Select Dates…"));
-        gtk_entry_set_text (GTK_ENTRY (popover->date_entry), "");
+        gtk_editable_set_text (GTK_EDITABLE (popover->date_entry), "");
         gtk_widget_hide (popover->clear_date_button);
     }
 }
@@ -725,12 +751,7 @@ nautilus_search_popover_closed (GtkPopover *popover)
     /* Reselect today at the calendar */
     g_signal_handlers_block_by_func (self->calendar, calendar_day_selected, self);
 
-    gtk_calendar_select_month (GTK_CALENDAR (self->calendar),
-                               g_date_time_get_month (now) - 1,
-                               g_date_time_get_year (now));
-
-    gtk_calendar_select_day (GTK_CALENDAR (self->calendar),
-                             g_date_time_get_day_of_month (now));
+    gtk_calendar_select_day (GTK_CALENDAR (self->calendar), now);
 
     g_signal_handlers_unblock_by_func (self->calendar, calendar_day_selected, self);
 }
@@ -926,21 +947,21 @@ nautilus_search_popover_init (NautilusSearchPopover *self)
     filter_time_type = g_settings_get_enum (nautilus_preferences, "search-filter-time-type");
     if (filter_time_type == NAUTILUS_QUERY_SEARCH_TYPE_LAST_MODIFIED)
     {
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->last_modified_button), TRUE);
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->last_used_button), FALSE);
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->created_button), FALSE);
+        gtk_check_button_set_active (GTK_CHECK_BUTTON (self->last_modified_button), TRUE);
+        gtk_check_button_set_active (GTK_CHECK_BUTTON (self->last_used_button), FALSE);
+        gtk_check_button_set_active (GTK_CHECK_BUTTON (self->created_button), FALSE);
     }
     else if (filter_time_type == NAUTILUS_QUERY_SEARCH_TYPE_LAST_ACCESS)
     {
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->last_modified_button), FALSE);
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->last_used_button), TRUE);
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->created_button), FALSE);
+        gtk_check_button_set_active (GTK_CHECK_BUTTON (self->last_modified_button), FALSE);
+        gtk_check_button_set_active (GTK_CHECK_BUTTON (self->last_used_button), TRUE);
+        gtk_check_button_set_active (GTK_CHECK_BUTTON (self->created_button), FALSE);
     }
     else
     {
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->last_modified_button), FALSE);
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->last_used_button), FALSE);
-        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->created_button), TRUE);
+        gtk_check_button_set_active (GTK_CHECK_BUTTON (self->last_modified_button), FALSE);
+        gtk_check_button_set_active (GTK_CHECK_BUTTON (self->last_used_button), FALSE);
+        gtk_check_button_set_active (GTK_CHECK_BUTTON (self->created_button), TRUE);
     }
 
     self->fts_enabled = g_settings_get_boolean (nautilus_preferences,
